import logging
from PyQt5.QtCore import pyqtSignal, Qt

from .textlist import TextlistModel, TextlistView


logger = logging.getLogger(__name__)


class CollectionsModel(TextlistModel):
    def data(self, index, role=Qt.DisplayRole):
        row = index.row()
        item = self._items[row]
        if role == Qt.DisplayRole:
            return '◎  ' + item.name
        return super().data(index, role)


class CollectionsView(TextlistView):
    show_collection = pyqtSignal([object])

    def __init__(self, parent):
        super().__init__(parent)

        self.clicked.connect(self._on_clicked)

    def _on_clicked(self, index):
        collection = index.data(role=Qt.UserRole)
        self.show_collection.emit(collection)



import logging

from PyQt5.QtCore import (
    QAbstractListModel,
    Qt,
    QModelIndex,
    QSize,
    QVariant,
)
from PyQt5.QtWidgets import QListView


logger = logging.getLogger(__name__)


class TextlistModel(QAbstractListModel):
    def __init__(self, parent):
        super().__init__(parent)
        self._items = []

    @property
    def items(self):
        return self._items

    def add(self, item):
        start = len(self._items)
        end = start + 1
        self.beginInsertRows(QModelIndex(), start, end)
        self._items.append(item)
        self.endInsertRows()

    def remove(self, item):
        row = self._items.index(item)
        self.beginRemoveRows(QModelIndex(), row, row + 1)
        self._items.remove(item)
        self.endRemoveRows()

    def clear(self):
        total_length = len(self.items)
        self.beginRemoveRows(QModelIndex(), 0, total_length - 1)
        self._items = []
        self.endRemoveRows()

    def rowCount(self, _=QModelIndex()):
        return len(self.items)

    def flags(self, index):  # pylint: disable=no-self-use
        if not index.isValid():
            return 0
        flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return flags

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        row = index.row()
        item = self.items[row]
        if role == Qt.UserRole:
            return item
        return QVariant()


class TextlistView(QListView):

    def __init__(self, parent):
        super().__init__(parent)

        self.setAttribute(Qt.WA_MacShowFocusRect, 0)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

    def wheelEvent(self, e):  # pylint: disable=no-self-use
        e.ignore()

    def sizeHint(self):
        height = 10
        for i in range(0, self.model().rowCount()):
            height += self.sizeHintForRow(i)
        return QSize(self.width(), height)

    def rowsInserted(self, parent, start, end):
        super().rowsInserted(parent, start, end)
        self.setFixedHeight(self.sizeHint().height())

    def rowsAboutToBeRemoved(self, parent, start, end):
        super().rowsAboutToBeRemoved(parent, start, end)
        self.setFixedHeight(self.sizeHint().height())



from PyQt5.QtCore import (
    pyqtSignal,
    Qt,
)
from PyQt5.QtWidgets import (
    QTableWidget,
    QTableWidgetItem,
    QWidget,
    QAbstractItemView,
    QHeaderView,
)


class CollectionItemsTable(QTableWidget):
    show_url = pyqtSignal([str])

    def __init__(self, parent):
        super().__init__(0, 2, parent)

        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.horizontalHeader().setSectionResizeMode(1, QHeaderView.Stretch)
        self.setHorizontalHeaderLabels(['地址', '描述'])
        self._setup_ui()

        self._items = []
        self.cellDoubleClicked.connect(
            lambda row, col: self.show_url.emit(self._items[row][0]))

    def _setup_ui(self):
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.horizontalHeader().setDefaultAlignment(Qt.AlignLeft | Qt.AlignVCenter)
        self.verticalHeader().hide()
        self.setShowGrid(False)
        self.setAlternatingRowColors(True)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.setColumnWidth(0, 300)

    def show_items(self, items):
        self.setRowCount(0)
        self._items = []
        for index, item in enumerate(items):
            row = index
            url, desc = item
            self.setRowCount(row + 1)
            url_item = QTableWidgetItem(url)
            desc_item = QTableWidgetItem(desc)
            self.setItem(row, 0, url_item)
            self.setItem(row, 1, desc_item)
        self._items = items
